@@ -26,9 +26,6 @@ extern crate alloc;
 #[cfg(not(feature = "std"))]
 use alloc::Vec;
 
-#[cfg(feature="heapsizeof")]
-extern crate heapsize;
-
 #[cfg(feature = "serde")]
 extern crate serde;
 
@@ -48,8 +45,6 @@ use std::ptr;
 use std::slice;
 #[cfg(feature = "std")]
 use std::io;
-#[cfg(feature="heapsizeof")]
-use std::os::raw::c_void;
 #[cfg(feature = "serde")]
 use serde::ser::{Serialize, Serializer, SerializeSeq};
 #[cfg(feature = "serde")]
@@ -57,8 +52,6 @@ use serde::de::{Deserialize, Deserializer, SeqAccess, Visitor};
 #[cfg(feature = "serde")]
 use std::marker::PhantomData;
 
-#[cfg(feature="heapsizeof")]
-use heapsize::{HeapSizeOf, heap_size_of};
 use SmallVecData::{Inline, Heap};
 
 /// Common operations implemented by both `Vec` and `SmallVec`.
@@ -607,33 +600,23 @@ impl<A: Array> SmallVec<A> {
         unsafe {
             let old_len = self.len;
             assert!(index <= old_len);
-            let mut ptr = self.as_mut_ptr().offset(index as isize);
-
-            // Move the trailing elements.
+            let ptr = self.as_mut_ptr().offset(index as isize);
             ptr::copy(ptr, ptr.offset(lower_size_bound as isize), old_len - index);
-
-            // In case the iterator panics, don't double-drop the items we just copied above.
-            self.set_len(index);
-
-            let mut num_added = 0;
-            for element in iter {
-                let mut cur = ptr.offset(num_added as isize);
-                if num_added >= lower_size_bound {
-                    // Iterator provided more elements than the hint.  Move trailing items again.
-                    self.reserve(1);
-                    ptr = self.as_mut_ptr().offset(index as isize);
-                    cur = ptr.offset(num_added as isize);
-                    ptr::copy(cur, cur.offset(1), old_len - index);
+            for (off, element) in iter.enumerate() {
+                if off < lower_size_bound {
+                    ptr::write(ptr.offset(off as isize), element);
+                    self.len = self.len + 1;
+                } else {
+                    // Iterator provided more elements than the hint.
+                    assert!(index + off >= index);  // Protect against overflow.
+                    self.insert(index + off, element);
                 }
-                ptr::write(cur, element);
-                num_added += 1;
             }
+            let num_added = self.len - old_len;
             if num_added < lower_size_bound {
                 // Iterator provided fewer elements than the hint
                 ptr::copy(ptr.offset(lower_size_bound as isize), ptr.offset(num_added as isize), old_len - index);
             }
-
-            self.set_len(old_len + num_added);
         }
     }
 
@@ -698,20 +681,6 @@ impl<A: Array> SmallVec<A> where A::Item: Copy {
     }
 }
 
-#[cfg(feature="heapsizeof")]
-impl<A: Array> HeapSizeOf for SmallVec<A> where A::Item: HeapSizeOf {
-    fn heap_size_of_children(&self) -> usize {
-        match self.data {
-            Inline { .. } => 0,
-            Heap { ptr, .. } => {
-                self.iter().fold(
-                    unsafe { heap_size_of(ptr as *const c_void) },
-                    |n, elem| n + elem.heap_size_of_children())
-            },
-        }
-    }
-}
-
 impl<A: Array> ops::Deref for SmallVec<A> {
     type Target = [A::Item];
     #[inline]
@@ -838,6 +807,20 @@ impl<'a, A: Array> From<&'a [A::Item]> for SmallVec<A> where A::Item: Clone {
     }
 }
 
+impl<A: Array> From<Vec<A::Item>> for SmallVec<A> {
+    #[inline]
+    fn from(vec: Vec<A::Item>) -> SmallVec<A> {
+        SmallVec::from_vec(vec)
+    }
+}
+
+impl<A: Array> From<A> for SmallVec<A> {
+    #[inline]
+    fn from(array: A) -> SmallVec<A> {
+        SmallVec::from_buf(array)
+    }
+}
+
 macro_rules! impl_index {
     ($index_type: ty, $output_type: ty) => {
         impl<A: Array> ops::Index<$index_type> for SmallVec<A> {
@@ -1130,11 +1113,6 @@ pub mod tests {
     #[cfg(not(feature = "std"))]
     use alloc::vec::Vec;
 
-    #[cfg(feature="heapsizeof")]
-    use heapsize::HeapSizeOf;
-    #[cfg(feature="heapsizeof")]
-    use std::mem::size_of;
-
     // We heap allocate all these strings so that double frees will show up under valgrind.
 
     #[test]
@@ -1398,36 +1376,6 @@ pub mod tests {
         assert_eq!(&v.iter().map(|v| *v).collect::<Vec<_>>(), &[0, 5, 6, 1, 2, 3]);
     }
 
-    #[test]
-    // https://github.com/servo/rust-smallvec/issues/96
-    fn test_insert_many_panic() {
-        struct PanicOnDoubleDrop {
-            dropped: Box<bool>
-        }
-
-        impl Drop for PanicOnDoubleDrop {
-            fn drop(&mut self) {
-                assert!(!*self.dropped, "already dropped");
-                *self.dropped = true;
-            }
-        }
-
-        struct BadIter;
-        impl Iterator for BadIter {
-            type Item = PanicOnDoubleDrop;
-            fn size_hint(&self) -> (usize, Option<usize>) { (1, None) }
-            fn next(&mut self) -> Option<Self::Item> { panic!() }
-        }
-
-        let mut vec: SmallVec<[PanicOnDoubleDrop; 1]> = SmallVec::new();
-        vec.push(PanicOnDoubleDrop { dropped: Box::new(false) });
-        vec.push(PanicOnDoubleDrop { dropped: Box::new(false) });
-        let result = ::std::panic::catch_unwind(move || {
-            vec.insert_many(0, BadIter);
-        });
-        assert!(result.is_err());
-    }
-
     #[test]
     #[should_panic]
     fn test_invalid_grow() {
@@ -1592,6 +1540,31 @@ pub mod tests {
     fn test_from() {
         assert_eq!(&SmallVec::<[u32; 2]>::from(&[1][..])[..], [1]);
         assert_eq!(&SmallVec::<[u32; 2]>::from(&[1, 2, 3][..])[..], [1, 2, 3]);
+
+        let vec = vec![];
+        let small_vec: SmallVec<[u8; 3]> = SmallVec::from(vec);
+        assert_eq!(&*small_vec, &[]);
+        drop(small_vec);
+
+        let vec = vec![1, 2, 3, 4, 5];
+        let small_vec: SmallVec<[u8; 3]> = SmallVec::from(vec);
+        assert_eq!(&*small_vec, &[1, 2, 3, 4, 5]);
+        drop(small_vec);
+
+        let vec = vec![1, 2, 3, 4, 5];
+        let small_vec: SmallVec<[u8; 1]> = SmallVec::from(vec);
+        assert_eq!(&*small_vec, &[1, 2, 3, 4, 5]);
+        drop(small_vec);
+
+        let array = [1];
+        let small_vec: SmallVec<[u8; 1]> = SmallVec::from(array);
+        assert_eq!(&*small_vec, &[1]);
+        drop(small_vec);
+
+        let array = [99; 128];
+        let small_vec: SmallVec<[u8; 128]> = SmallVec::from(array);
+        assert_eq!(&*small_vec, vec![99u8; 128].as_slice());
+        drop(small_vec);
     }
 
     #[test]
@@ -1600,30 +1573,6 @@ pub mod tests {
         assert_eq!(&SmallVec::<[u32; 2]>::from_slice(&[1, 2, 3][..])[..], [1, 2, 3]);
     }
 
-    #[cfg(feature="heapsizeof")]
-    #[test]
-    fn test_heap_size_of_children() {
-        let mut vec = SmallVec::<[u32; 2]>::new();
-        assert_eq!(vec.heap_size_of_children(), 0);
-        vec.push(1);
-        vec.push(2);
-        assert_eq!(vec.heap_size_of_children(), 0);
-        vec.push(3);
-        assert_eq!(vec.heap_size_of_children(), 16);
-
-        // Now check with reserved space
-        let mut vec = SmallVec::<[u32; 2]>::new();
-        vec.reserve(10);  // Rounds up to 16
-        assert_eq!(vec.heap_size_of_children(), 64);
-
-        // Check with nested heap structures
-        let mut vec = SmallVec::<[Vec<u32>; 2]>::new();
-        vec.reserve(10);
-        vec.push(vec![2, 3, 4]);
-        assert_eq!(vec.heap_size_of_children(),
-                   vec![2, 3, 4].heap_size_of_children() + 16 * size_of::<Vec<u32>>());
-    }
-
     #[test]
     fn test_exact_size_iterator() {
         let mut vec = SmallVec::<[u32; 2]>::from(&[1, 2, 3][..]);
